.AppBar{display:-webkit-flex;display:flex;-webkit-align-items:center;align-items:center;min-height:2.3rem;border-bottom:.0625rem solid #dceafb;border-left:.0625rem solid #e8f1fc;border-right:.0625rem solid #e8f1fc;padding-left:1.5rem}.AppBar__title{-webkit-flex:1 1;flex:1 1;text-transform:uppercase;letter-spacing:.07em}.AppBar__content{display:-webkit-flex;display:flex;-webkit-align-items:center;align-items:center}.NotificationContainer .MuiAlert-outlinedSuccess{border:.0625rem solid #2bc784}.NotificationContainer .MuiAlert-outlinedError{border:.0625rem solid #e64e48}.NotificationContainer .MuiPaper-root{background:#fff;height:4rem;display:-webkit-flex;display:flex;-webkit-align-items:center;align-items:center;border-radius:.375rem}.NotificationContainer .MuiPaper-root .MuiAlert-icon{padding:0}.NotificationContainer .MuiPaper-root .MuiAlert-icon img{width:2.375rem;height:2.375rem}.NotificationContainer .MuiPaper-root .MuiAlert-action{display:-webkit-flex;display:flex;-webkit-align-items:flex-start;align-items:flex-start;height:100%}.NotificationContainer .MuiPaper-root .MuiAlert-action .MuiSvgIcon-fontSizeSmall{font-size:1.05rem;fill:#1c2852}.NotificationContainer__contentBox__severity{font-weight:600;font-size:.875rem;text-transform:capitalize;color:#1c2852}.NotificationContainer__contentBox__message{font-weight:400;font-size:.875rem;color:#1c2852}.ConfirmModal .MuiBackdrop-root{background-color:rgba(0,0,0,.4)}.ConfirmModal .MuiPaper-rounded{border-radius:6px}.ConfirmModal .MuiPaper-elevation10{box-shadow:0 2px 4px #767a80}.ConfirmModal__Body{min-width:384px;padding:24px}.ConfirmModal__Body .ConfirmModal__Title__Container{display:-webkit-flex;display:flex;-webkit-align-items:center;align-items:center;margin-bottom:12px}.ConfirmModal__Close__Icon{position:absolute;right:10px;top:10px}.ConfirmModal__Icon{display:-webkit-flex;display:flex;-webkit-justify-content:center;justify-content:center;-webkit-align-items:center;align-items:center;width:32px;height:32px;margin-right:16px;border-radius:50%}.ConfirmModal__Icon .Icon__container{font-size:14px}.ConfirmModal__CancelButton{margin-right:10px;color:#1c2852}.ConfirmModal__Footer{padding:16px;margin:0 8px;border-top:1px solid #dee6f3;display:-webkit-flex;display:flex;-webkit-justify-content:end;justify-content:end;-webkit-align-items:center;align-items:center}.ConfirmModal__error .ConfirmModal__Icon{background-color:#fdeded}.ConfirmModal__error .ConfirmModal__Icon .Icon__container{color:#e64e48}.ConfirmModal__error .ConfirmModal__ConfirmButton{background-color:#e64e48}.ConfirmModal__warning .ConfirmModal__Icon{background-color:#fffae6}.ConfirmModal__warning .ConfirmModal__Icon .Icon__container{color:#fc0}.ConfirmModal__warning .ConfirmModal__ConfirmButton{background-color:#fc0}.ConfirmModal__success .ConfirmModal__Icon{background-color:#eaf9f3}.ConfirmModal__success .ConfirmModal__Icon .Icon__container{color:#2bc784}.ConfirmModal__success .ConfirmModal__ConfirmButton{background-color:#2bc784}.ConfirmModal__info .ConfirmModal__Icon{background-color:#e8f1fc}.ConfirmModal__info .ConfirmModal__Icon .Icon__container{color:#1473e6}.ConfirmModal__info .ConfirmModal__ConfirmButton{background-color:#1473e6}.CodeBlock{background:#f2f3f4;border-radius:.375rem;padding:1em 3em 1em 1em;position:relative;min-height:3.5rem;display:-webkit-flex;display:flex;-webkit-align-items:center;align-items:center}.CodeBlock pre{margin:0;font-style:normal;font-weight:400;font-size:.875rem;line-height:1.3125em;color:#031123;overflow:auto}.CodeBlock .prism-code{background-color:transparent!important}.CodeBlock__copy__span{top:1em;right:1em;width:1.5rem;height:1.5rem;color:#414b6d;display:-webkit-flex;display:flex;cursor:pointer;position:absolute;-webkit-align-items:center;align-items:center;-webkit-justify-content:center;justify-content:center}.BookmarkCard{position:relative;display:-webkit-flex;display:flex;-webkit-flex-direction:column;flex-direction:column;border:.0625rem solid #dee6f3;border-radius:.5rem;padding:1.5rem;margin-bottom:1rem}.BookmarkCard__delete{margin-left:.25rem}.BookmarkCard__top{-webkit-flex:1 1;flex:1 1}.BookmarkCard__top p{margin-top:.25rem;font-weight:400;font-size:.75rem}.BookmarkCard__title__section{-webkit-justify-content:space-between;justify-content:space-between}.BookmarkCard__title__section,.BookmarkCard__title__section__container{display:-webkit-flex;display:flex}.BookmarkCard__title__section__container i{margin-right:.5rem;margin-top:.125rem}.BookmarkCard__title__section a{text-decoration:none}.BookmarkCard__bottom{margin-top:.75rem}.BookmarkCard__bottom strong{color:#243969;margin-right:.5rem}.BookmarkCard__selected__metrics{margin-top:.875rem;width:100%;white-space:nowrap;overflow:auto;padding:.125rem 0}.BookmarkCard__selected__metrics .Badge{margin-right:.5rem}.BookmarkCard__run__expression{margin-right:2.8125rem}.Bookmarks{height:100vh}.Bookmarks__title{margin:.5em 0;padding:0;color:#1473e6}.Bookmarks__list{padding:1.5em}
/*# sourceMappingURL=bookmarks.css?version=93775a23a6452a4c7be3.map */